<?php
/**
 * Checkout card expiration fields class
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Checkout\Form_Field;

use WC_Asaas\Gateway\Gateway;
use WC_Asaas\Helper\Credit_Card_Helper;
use WC_Asaas\Helper\Validation_Helper;

/**
 * Checkout card expiration fields
 */
class Card_Number extends Form_Field {

	/**
	 * The unique WooCommerce field type
	 *
	 * @var string
	 */
	public function get_type() {
		return 'asaas-card-number';
	}

	/**
	 * Get the card field
	 *
	 * @param string $key The field key.
	 * @param array  $args The field args.
	 * @return array The subfield list.
	 */
	public function get_subfields( $key, $args ) {
		$args['type']              = 'text';
		$args['custom_attributes'] = array(
			'data-mask' => '0000 0000 0000 0000',
		);

		return apply_filters(
			'woocommerce_asaas_card_number_subfields', array(
				$key . '_number' => $args,
			), $key
		);
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @param Gateway $gateway The gateway is being processed.
	 * @param array   $key The field key.
	 * @param array   $field The field args.
	 * @param array   $data The payment form posted data.
	 */
	public function validate( $gateway, $key, $field, $data ) {
		$validation_helper = new Validation_Helper();
		$errors            = new \WP_Error();
		$card_helper       = new Credit_Card_Helper();

		foreach ( $this->get_subfields( $key, $field ) as $subkey => $subfield ) {
			$validation_helper->validate_required( $gateway, $subkey, $subfield, $data );
		}

		$card_number = $data[ $key . '_number' ];
		if ( ! empty( $card_number ) && ! $card_helper->get_card_brand( $gateway, $card_number ) ) {
			$brands_supported_names = $card_helper->supported_brands_name( $gateway );
			$errors->add( $gateway->get_error_code(), $this->invalid_card_message( $brands_supported_names ) );
		}

		$gateway->add_validation_errors( $errors );
		return empty( $errors->get_error_codes() );
	}

	/**
	 * Get invalid card message
	 *
	 * Use natural language conjuction to return the brand list.
	 *
	 * @link https://gist.github.com/angry-dan/e01b8712d6538510dd9c
	 *
	 * @param string[] $brands_supported_names The supported brands name.
	 * @return string The invalid card message.
	 */
	public function invalid_card_message( $brands_supported_names ) {
		$last          = array_pop( $brands_supported_names );
		$conjunction   = __( 'and', 'woo-asaas' );
		$brands_string = implode( ', ', $brands_supported_names ) . ' ' . $conjunction . ' ' . $last;

		return apply_filters(
			'woocommerce_asaas_invalid_brand_message',
			/* translators: %s: brands name */
			sprintf( __( 'The card number is invalid or you\'re trying to use a card brand that we don\'t accept . We just accept %s.', 'woo-asaas' ), $brands_string ),
			$brands_supported_names
		);
	}

	/**
	 * Process card number
	 *
	 * @param string         $key The field key.
	 * @param array          $field The field args.
	 * @param string         $data The field data.
	 * @param boolean|string $field_name The field name. Default: The key arg.
	 */
	public function process_data( $key, $field, &$data, $field_name = false ) {
		$field_name = false === $field_name ? $key : $field_name;
		$subfields  = $this->get_subfields( $key, $field );

		foreach ( $subfields as $subkey => $subfield ) {
			$subfield_data = isset( $_POST[ $subkey ] ) ? sanitize_text_field( wp_unslash( $_POST[ $subkey ] ) ) : ''; // phpcs:ignore WordPress.CSRF.NonceVerification.NoNonceVerification
			$subfield_data = preg_replace( '/\s/', '', $subfield_data );

			$data[ $subkey ] = $subfield_data;
		}
	}
}
