<?php
/**
 * Plugin main class
 *
 * @package WooAsaas
 */

namespace WC_Asaas;

use WC_Asaas\Admin\Plugin_Dependency;
use WC_Asaas\Webhook\Endpoint;
use WC_Asaas\Gateway\Gateway;
use WC_Asaas\Checkout\Form_Field\Card_Expiration;
use WC_Asaas\Checkout\Form_Field\Label;
use WC_Asaas\Checkout\Form_Field\Card;
use WC_Asaas\Checkout\Form_Field\One_Click_Options;
use WC_Asaas\Checkout\Form_Field\Card_Number;
use WC_Asaas\Checkout\Form_Field\Card_Security_Code;
use WC_Asaas\Billing_Type\Billing_Type_Exception;

/**
 * Asaas Gateway for WooCommerce main class
 */
class WC_Asaas {

	/**
	 * WooCommerce version.
	 *
	 * @var string
	 */
	public $version = '1.3.5';

	/**
	 * Instance of this class
	 *
	 * @var self
	 */
	protected static $instance = null;

	/**
	 * Gateway list with id
	 *
	 * @var Gateway[string]
	 */
	protected $gateways;

	/**
	 * Initialize the plugin public actions
	 *
	 * Block external object instantiation.
	 */
	private function __construct() {
		add_action( 'init', array( $this, 'load_plugin_textdomain' ) );
		
		$dependency = Plugin_Dependency::get_instance();
		add_action( 'admin_init', array( $dependency, 'check_dependencies' ) );

		if ( ! $dependency->check_woocommerce() ) {
			return;
		}

		add_action( 'init', array( $this, 'init_gateways' ) );
		add_action( 'init', array( Endpoint::get_instance(), 'custom_rewrite_basic' ) );
		add_action( 'init', array( $this, 'init_form_fields' ) );
		add_action( 'init', array( $this, 'schedule_remove_expired_ticket' ) );
		add_action( 'remove_expired_ticket', array( $this, 'remove_expired_ticket' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_frontend_files' ) );
		add_filter( 'woocommerce_payment_gateways', array( $this, 'register_gateways' ) );

		add_action( 'admin_notices', array( $this, 'check_checkout_settings' ) );
	}

	/**
	 * Return an instance of this class
	 *
	 * @return self A single instance of this class.
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Get the plugin absolute path
	 *
	 * @return string The plugin absolute path.
	 */
	public function get_plugin_path() {
		return plugin_dir_path( dirname( __FILE__ ) );
	}

	/**
	 * Get the plugin URL
	 *
	 * @return string The plugin URL.
	 */
	public function get_plugin_url() {
		return plugin_dir_url( dirname( __FILE__ ) );
	}

	/**
	 * Get templates path
	 */
	public function get_templates_path() {
		return $this->get_plugin_path() . 'templates/';
	}

	/**
	 * Get a template file
	 *
	 * A template can be overwritten by the theme. Add the templates file in `woocommerce/asaas` in your theme.
	 *
	 * @param string  $template_name The template file name.
	 * @param array   $args The template variables.
	 * @param boolean $return Return the template HTML if is true. Otherwise, print the template.
	 * @return string
	 */
	public function get_template_file( $template_name, $args = array(), $return = false ) {
		if ( $return ) {
			return wc_get_template_html(
				$template_name,
				$args,
				'woocommerce/asaas/',
				self::get_instance()->get_templates_path()
			);
		}

		wc_get_template(
			$template_name,
			$args,
			'woocommerce/asaas/',
			self::get_instance()->get_templates_path()
		);
	}

	/**
	 * Get plugin assets URL
	 */
	public function get_assets_url() {
		return $this->get_plugin_url() . 'assets/';
	}

	/**
	 * Init Asaas gateways together with WooCommerce to process the inner hooks correctly
	 */
	public function init_gateways() {
		foreach ( $this->get_gateways_classes() as $gateway ) {
			$reflection                         = new \ReflectionClass( $gateway );
			$gateway_obj                        = $reflection->newInstanceArgs();
			$this->gateways[ $gateway_obj->id ] = $gateway_obj;
		}
	}

	/**
	 * Load plugin textdomain file
	 */
	public function load_plugin_textdomain() {
		$domain                 = 'woo-asaas';
		$languages_rel_dir_path = '/../languages/';
		if ( 'development' === getenv( 'ENV' ) ) {
			load_textdomain( $domain, __DIR__ . $languages_rel_dir_path . '/' . $domain . '-' . get_locale() . '.mo' );
		} else {
			load_plugin_textdomain( $domain, false, basename( dirname( dirname( __FILE__ ) ) ) . '/languages' );
		}
	}

	/**
	 * Get plugin gateway classes
	 *
	 * @return string[] The fully qualified gateways classes name
	 */
	public function get_gateways_classes() {
		return apply_filters(
			'woocommerce_asaas_getways_classes', array(
				\WC_Asaas\Gateway\Ticket::class,
				\WC_Asaas\Gateway\Credit_Card::class,
			)
		);
	}

	/**
	 * Get plugin gateways
	 *
	 * @return Gateway[string] The plugin gateways associated with its id.
	 */
	public function get_gateways() {
		return apply_filters( 'woocommerce_asaas_getways', $this->gateways );
	}

	/**
	 * Get gateway by id
	 *
	 * @param  string $id id.
	 *
	 * @return Gateway return order gateway
	 */
	public function get_gateway_by_id( $id ) {
		return $this->gateways[ $id ];
	}

	/**
	 * Get gateway by billing type
	 *
	 * @param string $billing_type Billing type id on Asaas API.
	 * @throws Billing_Type_Exception If the type wasn't registered by a gateway.
	 * @return Gateway|NULL The gateway, if id is valid. Otherwise, null.
	 */
	public function get_gateway_by_billing_type( $billing_type ) {
		foreach ( $this->gateways as $gateway ) {
			if ( $billing_type === $gateway->get_type()->get_id() ) {
				return $gateway;
			}
		}

		/* translators: %s: billing type name  */
		throw new Billing_Type_Exception( sprintf( __( 'Billing type %s wasn\'t registered.', 'woo-asaas' ), $billing_type ) );
	}

	/**
	 * Add plugin ticket and credit card gateways to WooCommerce
	 *
	 * @param string[] $methods WooCommerce available gateways.
	 * @return string[] Gateways including Asaas ones.
	 */
	public function register_gateways( $methods ) {
		return array_merge( $methods, array_values( $this->gateways ) );
	}

	/**
	 * Init the form fields
	 *
	 * @see self::get_form_fields()
	 */
	public function init_form_fields() {
		$this->get_form_fields();
	}

	/**
	 * Get the custom checkout form fields
	 *
	 * @return \WC_Asaas\Checkout\Form_Field\Form_Field[]
	 */
	public function get_form_fields() {
		return array(
			Card::get_instance(),
			Card_Expiration::get_instance(),
			Card_Number::get_instance(),
			Card_Security_Code::get_instance(),
			Label::get_instance(),
			One_Click_Options::get_instance(),
		);
	}

	/**
	 * Get the custom checkout form field from object type
	 *
	 * @param string $type The type identification.
	 * @return \WC_Asaas\Checkout\Form_Field\Form_Field|NULL The correspondent object of the type. Null, if not found it.
	 */
	public function get_form_field_object_from_type( $type ) {
		foreach ( $this->get_form_fields() as $field_type ) {
			if ( $type === $field_type->get_type() ) {
				return $field_type;
			}
		}

		return null;
	}

	/**
	 * Check checkout settings for 'woocommerce-extra-checkout-fields-for-brazil' plugin
	 */
	public function check_checkout_settings() {
		$settings = get_option( 'wcbcf_settings' );
		
		if ( is_array( $settings ) ) {
			if ( $settings['person_type'] == '0' ) {	
				$error = __( 'Asaas needs the CPF or CNPJ at checkout for the integration works.', 'woo-asaas' );
				$message = $error;

				if ( 'woocommerce-extra-checkout-fields-for-brazil' != $_GET['page'] ) {
					$message .= ' ' . sprintf( __( '<a href="%s">Click here</a> to change checkout fields.', 'woo-asaas' ), admin_url( 'admin.php?page=woocommerce-extra-checkout-fields-for-brazil' ) );
				}

				echo '<div class="error"><p>' . $message . '</p></div>';
			}
		}
	}

	/**
	 * Enqueue plugin style file
	 *
	 * If SCRIPT_DEBUG is active, load the non minified file.
	 */
	public function enqueue_frontend_files() {
		$min = '.min';

		if ( defined( 'SCRIPT_DEBUG' ) && true === SCRIPT_DEBUG ) {
			$min = '';
		}

		wp_enqueue_style( 'woo-asaas', $this->get_plugin_url() . 'assets/css/woo-asaas' . $min . '.css', array(), $this->version );
		wp_enqueue_script( 'woo-asaas', $this->get_plugin_url() . 'assets/js/woo-asaas' . $min . '.js', array(), $this->version, true );
	}

	/**
	 * Create a daily cron event, if one does not already exist.
	 * 
	 * Scheduling to remove overdue ticket.
	 */
	public function schedule_remove_expired_ticket() {
		if ( ! wp_next_scheduled( 'remove_expired_ticket' ) ) {
			wp_schedule_event( time(), 'daily', 'remove_expired_ticket', array() );
		}
	}

	/**
	 * Executes the call to remove overdue tickets.
	 * 
	 * @return string Response message with removal result.
	 */
	public function remove_expired_ticket() {
		$ticket = new \WC_Asaas\Gateway\Ticket;
		return $ticket->remove_expired_ticket();
	}
}
