<?php
/**
 * File for class Webhook
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Webhook;

use WC_Asaas\Gateway\Gateway;

/**
 * Webhook
 */
class Webhook {

	/**
	 * Attributes of the webhook
	 *
	 * @var object $data
	 */
	private $data;

	/**
	 * Order related to webhook
	 *
	 * @var \WC_Order
	 */
	private $order;

	/**
	 * Discounts values
	 *
	 * @var float
	 */
	private $discounts_values;

	const PREFIX_LOG = 'Asaas: ';

	const PAYMENT_CREATED = 'PAYMENT_CREATED';

	const PAYMENT_UPDATED = 'PAYMENT_UPDATED';

	const PAYMENT_CONFIRMED = 'PAYMENT_CONFIRMED';

	const PAYMENT_RECEIVED = 'PAYMENT_RECEIVED';

	const PAYMENT_OVERDUE = 'PAYMENT_OVERDUE';

	const PAYMENT_REFUNDED = 'PAYMENT_REFUNDED';

	const PAYMENT_DELETED = 'PAYMENT_DELETED';

	const PAYMENT_RESTORED = 'PAYMENT_RESTORED';

	const PERCENTAGE_CALCULUS_TYPE = 'PERCENTAGE';

	const FIXED_CALCULUS_TYPE = 'FIXED';

	const CREDIT_CARD_PAYMENT_TYPE = 'CREDIT_CARD';

	/**
	 * Initialize the object
	 *
	 * @param Gateway   $gateway The payment gateway.
	 * @param \WC_Order $order The webhook that will be processed.
	 * @param \stdClass $data The webhook data.
	 */
	public function __construct( Gateway $gateway, \WC_Order $order, \stdClass $data ) {
		$this->gateway = $gateway;
		$this->order   = $order;
		$this->data    = $data;

		$this->order->set_date_modified( new \WC_DateTime( date( 'Y-m-d H:i:s' ) ) );

		add_action( 'woocommerce_order_after_calculate_totals', array( $this, 'set_order_discount' ) );
	}

	/**
	 * Magic function to access attributes
	 *
	 * @param  string $name name of the attribute.
	 * @return mixed The value of the attribute.
	 */
	public function __get( $name ) {
		return $this->data->{$name};
	}

	/**
	 * Process the event according to its type
	 *
	 * @throws \Exception Case event not found.
	 */
	public function process_event() {
		switch ( $this->event ) {
			case Webhook::PAYMENT_CONFIRMED:
				$this->on_payment_confirmed();
				break;

			case Webhook::PAYMENT_CREATED:
				$this->on_payment_created();
				break;

			case Webhook::PAYMENT_DELETED:
				$this->on_payment_deleted();
				break;

			case Webhook::PAYMENT_OVERDUE:
				$this->on_payment_overdue();
				break;

			case Webhook::PAYMENT_RECEIVED:
				$this->on_payment_received();
				break;

			case Webhook::PAYMENT_REFUNDED:
				$this->on_payment_refunded();
				break;

			case Webhook::PAYMENT_RESTORED:
				$this->on_payment_restored();
				break;

			case Webhook::PAYMENT_UPDATED:
				$this->on_payment_updated();
				break;

			default:
				/* translators: %s: event name  */
				die( esc_html( sprintf( __( 'Untreated event: %s', 'woo-asaas' ), $this->event ) ) );
		}
	}

	/**
	 * Method used when payment is confirmed
	 */
	private function on_payment_confirmed() {
		if ( $this->should_process_payment_on_payment_confirmed() ) {
			$this->process_payment();
		}

		$this->add_order_note( __( 'Payment confirmed.', 'woo-asaas' ) );
	}

	/**
	 * Process payment on confirmed event when is ticket or the option to
	 * antecipate credit card payment is checked.
	 *
	 * @return boolean
	 */
	private function should_process_payment_on_payment_confirmed() {
		if( self::CREDIT_CARD_PAYMENT_TYPE === $this->payment->billingType ) {
			return 'yes' === $this->gateway->get_option('payment_confirmed_status');
		}

		return true;
	}

	private function process_payment() {
		$order_data = $this->order->get_data();

		$paid_statuses = wc_get_is_paid_statuses();
		if ( $this->order->has_status( $paid_statuses ) ) {
			throw new Inconsistency_Data_Exception( esc_html__( 'This order was already paid.', 'woo-asaas' ) );
		}

		// Value for change total of order.
		if ( $order_data['total'] !== $this->payment->value && self::CREDIT_CARD_PAYMENT_TYPE === $this->payment->billingType ) {
			$this->update_order_values_on_credit_card();
		}

		$this->order->payment_complete( $this->payment->id );
	}

	/**
	 * Method used when payment is created
	 */
	private function on_payment_created() {
		$this->order->update_status( 'pending' );
		$this->add_order_note( __( 'Payment created.', 'woo-asaas' ) );
	}

	/**
	 * Method used when payment is deleted
	 */
	private function on_payment_deleted() {
		$this->order->update_status( 'cancelled' );
		$this->add_order_note( __( 'Payment deleted.', 'woo-asaas' ) );
	}

	/**
	 * Method used when payment is overdue
	 */
	private function on_payment_overdue() {
		$this->order->update_status( 'failed' );
		$this->add_order_note( __( 'Payment overdue.', 'woo-asaas' ) );
	}

	/**
	 * Method used when payment is received
	 */
	private function on_payment_received() {
		if ( $this->should_process_payment_on_payment_received() ) {
			$this->process_payment();
		}

		$this->add_order_note( 'Payment received.', 'woo-asaas' );
	}

	/**
	 * Process payment on payment event when is ticket and the payment wasn't
	 * completed or the option to antecipate credit card payment is unchecked.
	 *
	 * @return boolean
	 */
	private function should_process_payment_on_payment_received() {
		if( self::CREDIT_CARD_PAYMENT_TYPE === $this->payment->billingType ) {
			return 'yes' !== $this->gateway->get_option( 'payment_confirmed_status' );
		}

		return $this->payment->id !== $this->order->get_transaction_id();
	}

	private function update_order_values_on_credit_card() {
		if( $this->payment->value > $this->payment->originalValue ) {
			// Add tax for fine.
			if( isset( $this->payment->fine->value ) ) {
				$item_total = $this->calculate_item_total( $this->payment->fine, $this->payment->originalValue );

				$item_name = __( 'Fine tax', 'woo-asaas' );
				$this->add_item_fee_to_order( $item_name, $item_total );
			}

			// Add tax for interest.
			if( isset( $this->payment->interest->value ) ) {
				$item_total = $this->calculate_item_total( $this->payment->interest, $this->payment->originalValue );

				$item_name = __( 'Interest', 'woo-asaas' );
				$this->add_item_fee_to_order( $item_name, $item_total );
			}
		}

		if ( $this->payment->value < $this->payment->originalValue ) {
			// Add order discount.
			if ( isset( $this->payment->discount->value ) ) {
				$item_total = $this->calculate_item_total( $this->payment->discount, $this->payment->originalValue );

				$this->discounts_values = $item_total;
			}
		}

		// Calculate totals order.
		$this->order->calculate_totals();
	}

	private function calculate_item_total( $item_object, $total ) {
		$fine_type = self::FIXED_CALCULUS_TYPE;
		if( isset( $item_object->type ) ) {
			$fine_type = $item_object->type;
		}

		$item_total = $this->calculate_item_fee_value( $total, $fine_type, $item_object->value );

		return $item_total;
	}

	private function calculate_item_fee_value( $current_value, $type, $amount ) {
		if( self::PERCENTAGE_CALCULUS_TYPE === $type ) {
			return ( $current_value * ( $amount / 100 ) );
		}

		return $amount;
	}

	private function add_item_fee_to_order( $name, $total ) {
		$item_fee = new \WC_Order_Item_Fee();
		$item_fee->set_name( $name );
		$item_fee->set_total( $total );
		$this->order->add_item( $item_fee );
	}

	/**
	 * Method used when payment is refunded
	 */
	private function on_payment_refunded() {
		$this->order->update_status( 'refunded' );
		$this->add_order_note( 'Payment refunded.', 'woo-asaas' );
	}

	/**
	 * Method used when payment is restored
	 */
	private function on_payment_restored() {
		$this->order->update_status( 'pending' );
		$this->add_order_note( 'Payment restored.', 'woo-asaas' );
	}

	/**
	 * Method used when payment is updated
	 */
	private function on_payment_updated() {
		$this->order->set_total( $this->payment->value );
		$this->order->update_status( 'pending' );
		$this->add_order_note( 'Payment updated', 'woo-asaas' );
	}

	/**
	 * Discount should be applied after calculate order totals
	 */
	public function set_order_discount() {
		if( null === $this->discounts_values ) {
			return;
		}

		$order_total = $this->order->get_total() - $this->discounts_values;

		$this->order->set_discount_total( $this->discounts_values );
		$this->order->set_total( $order_total );
	}

	/**
	 * Add note
	 *
	 * @param String $note note description.
	 */
	private function add_order_note( $note ) {
		$this->order->add_order_note( self::PREFIX_LOG . ' ' . $note );
	}
}
